import papermill as pm

import logging
logging.basicConfig(format='OpenTLDR Logging: %(message)s')
_log=logging.getLogger("OpenTLDR")

# Pull environment variables from .env or os
from dotenv import load_dotenv
load_dotenv()

# disable warning for debugging of frozen packages
# seems to only matter when running under Jupyter
import os
os.environ["PYDEVD_DISABLE_FILE_VALIDATION"]="1"



class Workflow:

    # defaults can be overridden when creating workflow instance
    output_path:str = "./READ_ONLY_OUTPUT"
    workflow:list[str] = [
            "Step_0_Initialize.ipynb",
            "Step_1_Ingest.ipynb",
            "Step_1a_MockUI.ipynb",
            "Step_2_Connect.ipynb",
            "Step_3_Recommend.ipynb",
            "Step_4_Summarize.ipynb",
            "Step_5_Produce.ipynb",
            "Step_6_Evaluate.ipynb"
            ]
    parameters:dict = {
        "message":"Successfully passed parameters!"
        }

    def __init__(self, workflow:list[str], parameters:dict, output_path=None, database=None):
        if output_path is not None:
            self.output_path=output_path
        if workflow is not None:
            self.workflow=workflow
        if parameters is not None:
            self.parameters=parameters
        if database is not None:
            self.database=database
        self.ensure_path(self.output_path)

    def ensure_path(self,path):
        '''
        ensure_path(path) simply creates a directory if it is not there already.
        '''
        if not os.path.exists(path):
            os.makedirs(path)
            print("Created output directory: "+path)

    def execute_notebook(self, inNotebook:str):
        '''
        execute_notebook runs a single notebook file in PaperMill
        '''
        outNotebook:str = os.path.join(self.output_path,inNotebook.replace('/',"_"))
        pm.execute_notebook( inNotebook, outNotebook,
                         parameters=self.parameters)
    
    def run(self):
        '''
        run the defined workflow files in order in PaperMill
        '''
        for step in self.workflow:
            print("\nStarting "+step)
            self.execute_notebook(step)
            print("Completed "+step)



import unittest

import sys
sys.path.insert(0, 'src')

from opentldr import KnowledgeGraph as kg
from opentldr.Domain import ReferenceNode

class TestKg(unittest.TestCase):
    def test_default_connection(self):
        self.assertTrue(kg.connect())

    def test_run_query(self):
        rn= kg.add_reference_node(ReferenceNode(text="this is a test",type="TEST"))
        list = kg.cypher_query_to_list("MATCH (n:ReferenceNode) return n","n")
        self.assertTrue(rn in list)

        kg.delete_reference_node(rn)
        list = kg.cypher_query_to_list("MATCH (n:ReferenceNode) return n","n")
        self.assertFalse(rn in list)


unittest.main()


import neomodel as nm
from neomodel.cardinality import One, ZeroOrOne, ZeroOrMore, OneOrMore

import logging
logging.basicConfig(format='OpenTLDR Logging: %(message)s')
_log=logging.getLogger("OpenTLDR")

class OpenTldrMeta():
    '''
    OpenTldrMeta is a mixin class that adds a unique id (uid) and JSON property (metadata)
    to a Node or Edge. This is used in every Edge and Node in OpenTLDR and provides an
    interface for PyDantic for data manipulation and JSON.
    '''
    uid=nm.UniqueIdProperty()
    metadata=nm.JSONProperty(default="{}")
    
    def to_text(self) -> str:
        '''
        to_text() returns a user readable string representing the values in the object.
        '''
        return "uid: {uid}".format(uid=self.uid)
    
class OpenTldrText():
    '''
    OpenTldrText is a mixin class that adds the properties of text and type (both are Strings)
    to any Node or Edge. This is used whenever the client stores text in the knowledge graph.
    '''
    text = nm.StringProperty(required=True)
    type = nm.StringProperty(required=True)

    def to_text(self) -> str:
        return "type: {type}\t text: {text}".format(type=self.type,text=self.text)

class CitableNode(nm.StructuredNode):
    '''
    CitableNode is a baseclass for any Node from which we extract entities (e.g., NER)
    and we need to refer back to the Node from where they were detected.
    '''
    pass

class Uncertain():
    '''
    Uncertain is a mixin class that adds a confidence value (0.0-1.0) to a Node or Edge.
    This is implemented to store the float value and generate description qualatiative
    text clauses for the ranges of this confidence value.
    '''
    confidence = nm.FloatProperty(default=1.0)
    def uncertainty_to_text(self) -> str:
        if self.confidence < 0.05:
            return "with almost no chance"
        if self.confidence >= 0.05 and self.confidence < 0.20:
            return "very unlikely"
        if self.confidence >= 0.2 and self.confidence < 0.45:
            return "unlikely"
        if self.confidence >= 0.45 and self.confidence < 0.55:
            return "with roughly even chance"
        if self.confidence >= 0.55 and self.confidence < 0.8:
            return "likely"
        if self.confidence >= 0.8 and self.confidence < 0.95:
            return "very likely"
        if self.confidence >= 0.55 and self.confidence < 1.0:
            return "with almost certainty"
        return "with certainty"

class Scored():
    '''
    Scored is a mixin class that adds a score value (0.0-1.0) to a Node or Edge.
    This is implemented to store the float value and generate descriptive qualatative
    text clauses for the ranges of this score value.
    '''
    score = nm.FloatProperty(default=1.0)
    def score_to_text(self) -> str:
        if self.score < 0.15:
            return "very low"
        if self.score >= 0.15 and self.score < 0.40:
            return "low"
        if self.score >= 0.4 and self.score < 0.6:
            return "medium"
        if self.score >= 0.6 and self.score < 0.85:
            return "high"
        if self.score >= 0.85 and self.score < 1.0:
            return "very high"
        return "perfect"

# Reference Knowledge

class ReferenceEdge(nm.StructuredRel, OpenTldrText, OpenTldrMeta):
    '''
    ReferenceEdge allows for general reference data relationships to be introduced into the 
    knowledge graph and used abstractly (e.g., ontological distance using path finding).
    This uses the OpenTldrText mixin to implement text and type properties.
    '''
    hypothesized = nm.BooleanProperty(default=False)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def is_hypothesized(self) -> bool:
        return self.hypothesized

    def set_hypothesized(self, is_hypothesized:bool):
        self.hypothesized=is_hypothesized

    def to_text(self) -> str:
        left:ReferenceNode = self.start_node()
        right:ReferenceNode = self.end_node()
        return "{left} -[{relationship}:{type}]-> {right}".format(left=left.text,
                                                           right=right.text,
                                                           relationship=self.text,
                                                           type=self.type)


class ReferenceNode(nm.StructuredNode, OpenTldrText, OpenTldrMeta):
    '''
    ReferenceNode allos for general reference data nodes to be introduced into the 
    knowledge graph and used abstractly (e.g., to relate extracted entities to known things).
    This uses the OpenTldrText mixin to implement text and type properties.
    The hypothesized property is a boolean that indicates if this was inferred (e.g., discovered)
    or provided as fact via external reference data (e.g., imported). 
    '''
    hypothesized = nm.BooleanProperty(default=False)
    edges = nm.RelationshipTo('ReferenceNode','REFERENCE_EDGE', model=ReferenceEdge, cardinality=ZeroOrMore)

    def is_hypothesized(self) -> bool:
        return self.hypothesized

    def set_hypothesized(self, is_hypothesized:bool):
        self.hypothesized=is_hypothesized

    def connect_to(self, to_node:'ReferenceNode') -> ReferenceEdge:
        return self.edges.connect(to_node)

    def to_text(self) -> str:
        return "'{text}' of type {type}".format(text=self.text,type=self.type)
    

# Active Data

class Source(nm.StructuredNode, OpenTldrMeta):
    '''
    Source nodes indicate the generator of various content. This may be an author or distributor
    of information. The name property is used to uniquely identify the source.
    '''
    name = nm.StringProperty(unique_index=True, required=True)

class IsFrom(nm.StructuredRel, OpenTldrMeta):
    '''
    IsFrom edges connect Content nodesM to the Source nodes in the knowledge graph.
    '''
    def to_text(self) -> str:
        return "The {content_type} '{content_title}' is from the source {end}.".format(
            content_type= self.start_node().type,
            content_title=self.start_node().title,
            end=self.end_node().name)
    
class Content(CitableNode, OpenTldrMeta, OpenTldrText):
    '''
    Content nodes represent text-based information, such as a news article or message.
    They include properties for title (String), date (datetime.date), and url (String).
    In OpenTldr Content node urls are used for hyperlinks back to the original material.
    '''
    title = nm.StringProperty(required=True)
    date = nm.properties.DateProperty(required=True)
    url = nm.StringProperty(required=True)
    is_from = nm.RelationshipTo(Source, 'IS_FROM', model=IsFrom, cardinality=One)
    
class MentionedIn(nm.StructuredRel, OpenTldrMeta):
    '''
    MentionedIn edges connects an extracted Entity node to the CitableNode (e.g., Content 
    or Request) that included the reference.
    '''
    def to_text(self) -> str:
        return "The entity '{entity}' is mentioned in the '{title}'.".format(
            entity=self.start_node().text,
            title=self.end_node().title)

class RefersTo(nm.StructuredRel, OpenTldrMeta, Uncertain):
    '''
    RefersTo edges connects an extracted Entity node to the ReferenceNode that is it believed
    to represent with the uncertainity value provided. 
    '''
    def to_text(self) -> str:
        return "The entity '{entity}' {uncertainty} refers to '{ref}'.".format(
            entity=self.start_node().text,
            ref=self.end_node().text,
            uncertainty=self.uncertainty_to_text())

class Entity(nm.StructuredNode, OpenTldrText, OpenTldrMeta):
    '''
    Entity nodes represent specific things that are "mentioned_in" in Citable nodes 
    (e.g., Content or Requests) that "refers_to" some ReferenceNode information. The text property
    contains the text for the entity identified and they type property expresses what it is.
    The "refers_to" edge can be uncertain (e.g., sematic similarity of text and type consistency). 
    '''
    refers_to=nm.RelationshipTo(ReferenceNode,'REFERS_TO', model = RefersTo, cardinality=ZeroOrMore)  
    mentioned_in = nm.RelationshipTo("CitableNode", 'MENTIONED_IN', model=MentionedIn, cardinality=OneOrMore)

# Information Request

class User(nm.StructuredNode, OpenTldrMeta):
    '''
    User node represents a client of the OpenTLDR system and has name and email properties (Stings).
    Similar to a Content connecting to a Source, Requests connect to a User (see RequestedBy edge).
    '''
    name = nm.StringProperty(required=True)
    email = nm.StringProperty(required=True)
    def to_text(self) -> str:
        return "The User '{name}' with email '{email}'.".format(
            name=self.name, email=self.email)

class RequestedBy(nm.StructuredRel, OpenTldrMeta):
    '''
    RequestedBy edges connect Request nodes to the User node to reflect who made the request.
    '''
    def to_text(self) -> str:
        return "The request '{request}' was requested by {user}.".format(
            request=self.start_node().title,
            user=self.end_node().name)

class Request(CitableNode, OpenTldrMeta):
    '''
    Request nodes information requests that are made by users to indicate what information they
    are interested in and how to tailor that information for them. A "text" property (String) 
    holds the text of the information request and the "title" property (String) provides a 
    short labels for the request to avoid repeating long request text. The "requested_by" edge
    links each request to one User node.
    '''
    title=nm.StringProperty(required=True)
    text = nm.StringProperty(required=True)
    requested_by = nm.RelationshipTo(User, 'REQUESTED_BY', model=RequestedBy, cardinality=One)

# Workflow Products / Recommendations and Summaries

class Recommends(nm.StructuredRel, OpenTldrMeta):
    '''
    Recommends edge connects a Recommendation node to the Content node that is being recommended.
    '''
    def to_text(self) -> str:
        return "This has {score} relevance for {content_type} '{content_title}'.".format(
            content_title=self.end_node().title,
            content_type= self.end_node().type,
            score=self.start_node.score_to_text())

class RelatesTo(nm.StructuredRel, OpenTldrMeta):
    '''
    RelatesTo edge connects a Recommendation node to the Request on which it is based.
    '''
    def to_text(self) -> str:
        return "This has {score} relevance to the request '{request}'.".format(
            request= self.end_node().title,
            score=self.start_node().score_to_text())

class Recommendation(nm.StructuredNode, OpenTldrMeta, Scored):
    '''
    Recommendation nodes represent the assertion that a Content node (pointed to by Recommends edge)
    is believed to be relevant to a Request node (pointed to by the RelatesTo edge). The score
    property (float 0.0 - 1.0) indicates how relevant and thus how strong the recommendation.
    '''
    recommends = nm.RelationshipTo(Content, 'RECOMMENDS', model=Recommends, cardinality=One)
    relates_to = nm.RelationshipTo(Request, 'RELATES_TO', model=RelatesTo, cardinality=One)
    def to_text(self) -> str:
        return "The {content_type} '{content_title}' has {score} relevance to the request '{request}'.".format(
            content_title=self.recommends.single().text,
            content_type= self.recommends.single().type,
            request= self.relates_to.single().title,
            score=self.score_to_text())

class Summarizes(nm.StructuredRel, OpenTldrMeta):
    '''
    Summerizes edges connect a Summary node to the Content node that they summarize.
    '''
    pass

class FocusOn(nm.StructuredRel, OpenTldrMeta, Uncertain):
    '''
    FocusOn edges connect a Summary node to the Recommendation node that may help inform what
    the most relevant parts of the Summary are with respect to that Recommendation / Request.
    '''
    pass

class Summary(nm.StructuredNode, OpenTldrMeta):
    '''
    Summary node contains text (String property) that is a shortened (ideally tailored) version
    of the Content node connect with the "summaries" edge. The "focus_on" edge connects to the
    Recommendation that may inform how this summary is tailored.
    '''
    text = nm.StringProperty(required=True)
    summarizes = nm.RelationshipTo(Content, 'SUMMARIZES', model=Summarizes, cardinality=One)
    focus_on = nm.RelationshipTo(Recommendation, 'FOCUS_ON', model=FocusOn, cardinality=ZeroOrOne)

# Workflow Products / TLDR related classes

class Includes(nm.StructuredRel, OpenTldrMeta):
    '''
    Includes edges connect a TldrEntry node to a Summary node to indicate that the Summary is
    to be used in that part of the TLDR.
    '''
    pass

class BasedOn(nm.StructuredRel, OpenTldrMeta):
    '''
    BasedOn edges connect a TldrEntry node to a Recommendation ndoe to indicate why that
    entry is being included in the TLDR and may be used for things like ordering the entries
    to show the most recommended first.
    '''
    pass

class TldrEntry(nm.StructuredNode, OpenTldrMeta):
    '''
    TldrEntry is one record within a TLDR. In general, there is a one-to-one relationship
    between each TldrEntry, Recommendation, and Summary such that each record within a TLDR
    has a score for the Recommendation, a text blurb from the Summary, and a connection back
    to the original Content (from both the Recommendation and Summary).
    '''
    link = nm.StringProperty(required=False)
    includes = nm.RelationshipTo(Summary, 'INCLUDES', model=Includes, cardinality=One)
    based_on = nm.RelationshipTo(Recommendation, 'BASED_ON', model=BasedOn, cardinality=One)
    def to_text(self) -> str:
        return "The entry '{entry}' summarizes {content_type} '{content_title}' and {score} relevance to the request '{request}'.".format(
            entry=self.includes.single().summarizes.single().title,
            content_title=self.includes.single().summarizes.single().title,
            content_type= self.includes.single().summarizes.single().type,
            request= self.based_on.single().relates_to.single().title,
            score=self.based_on.single().score_to_text())

class Priority(nm.StructuredRel, OpenTldrMeta, Scored):
    '''
    Entries edges connect a Tldr to each of the TldrEntry objects that is make it up. The assumption
    here is that the score property (float 0.0 - 1.0) on the edge provides a decending ordering for
    the entries connected by these links.
    '''
    pass

class ResponseTo(nm.StructuredRel, OpenTldrMeta):
    '''
    ResponseTo edges connect a Tldr to the Request for which is was generated.
    '''
    pass

class Tldr(nm.StructuredNode, OpenTldrMeta):
    '''
    Tldr nodes represent an instance of the TLDR report for the date (in 'date' property of type
    datetime.date). It is connected to the Request by the ResponseTo edge and a set of TldrEntries
    by "Entries" edges.
    '''
    date = nm.DateProperty(required=True)
    priority = nm.RelationshipTo(TldrEntry, 'PRIORITY', model=Priority, cardinality=ZeroOrMore)
    response_to = nm.RelationshipTo(Request, 'RESPONSE_TO', model=ResponseTo, cardinality=One)
    def to_text(self) -> str:
        out:str = "The TLDR for request '{request}' on date {date} includes: ".format(
            request= self.response_to.single().title,
            date=self.date)
        for tldr_entry in self.priority:
            out += "\t{text}\n".format(text=tldr_entry.to_text())
        return out

# USER FEEDBACK / RATING


class Rated(nm.StructuredRel, Scored):
    '''
    Rated edges connect a Request node to TldrEntry node to provide a method for applying a User
    Feedback score ("score" property is float with expected range of 0.0 - 1.0) to indicating
    how relevant that entry is with respect to the request. These edges are generally created
    by the UI when the user rates an entry (e.g., clicking on a star rating).
    '''
    date = nm.DateProperty(required=True)
    def to_text(self) -> str:
        return "The user {user} rated the entry {entry} as {score}.".format(
            user=self.start_node().name,
            entry=self.end_node().text,
            score=self.score_to_text())

# USER FEEDBACK / ACCESS 

class Accessed(nm.StructuredRel, Scored):
    '''
    Accessed edges connect a User node to a Content node to indicate that the user has made
    a request (via the UI) for the original content (i.e., the url). The date property indicates
    when the request was made.
    '''
    date = nm.DateProperty(required=True)
    def to_text(self) -> str:
        return "The user {user} accessed the content {title} on {date}.".format(
            user=self.start_node().name,
            title=self.end_node().title,
            date=self.date)

#import jsonpickle
#
#
#def to_jsonpickle(obj) -> str:
#    return jsonpickle.encode(obj)
#
#def from_jsonpickle(json_string):
#    return jsonpickle.decode(json_string)


import os
from dotenv import load_dotenv
from abc import ABC, abstractmethod 
from datetime import datetime
import boto3

from opentldr import KnowledgeGraph
import opentldr.Domain as domain

load_dotenv()

import logging
logging.basicConfig(format='OpenTLDR Logging: %(message)s')
_log=logging.getLogger("OpenTLDR")

class AbstractContentRepo:
    kg:KnowledgeGraph=None

    @abstractmethod
    def importContentData(self) -> list[str]:
        pass

    @abstractmethod
    def describe(self) -> str:
        pass

    def importTextContent(self,
                          kg:KnowledgeGraph,
                          text:str,
                          url:str,
                          default_source_name:str="unknown",
                          title:str="untitled",
                          date:datetime.date=datetime.now().date(),
                          type:str="news",
                          author:str="unknown") -> domain.Content:
            '''
            importTextContent processes the lines in a text block, extracts some metadata, and stores it as
            Content nodes in the KnowledgeGraph. This is used by the implementations of the abstract class
            AbstractContentRepo once they retrieve text content from their source. All of the metadata can be
            passed into this method, but will be overwritten by that discovered within the text (if any).
            '''
            content_text=""
            source=None
                    
            for line in text.splitlines():
                if line.startswith("date:"):
                    content_date = datetime.strptime(line.split(":",1)[1].strip(),'%m/%d/%Y').date()
                elif line.startswith("title:"):
                    title = line.split(":",1)[1].strip()
                elif line.startswith("url:"):
                    url = line.split(":",1)[1].strip()
                elif line.startswith("type:"):
                    type = line.split(":",1)[1].strip()
                elif line.startswith("source:"):
                    source_name = line.split(":",1)[1].strip()
                    source=kg.get_source_by_name(name=source_name)
                    if source is None: # Identified source not in KG so add it
                        source=kg.add_source(name=source_name)
                elif line.startswith("author:"):
                    author = line.split(":",1)[1].strip()
                else:
                    content_text=content_text.join(["\n",line])
                
            # write Content node to KG
            if source is None: # No source specified in file
                source=kg.get_source_by_name(name=default_source_name)
                if source is None: # Default does not exist in KG yet
                    source=kg.add_source(name=default_source_name)

            content=kg.add_content(source=source, type=type, url=url, date=date, title=title, text=content_text.strip())
            return content

    def importTextRequest(self,
                          kg:KnowledgeGraph,
                          text:str,
                          default_user_name:str="unknown",
                          default_user_email:str="unknown",
                          title:str="untitled",
                          date:datetime.date=datetime.now().date()) -> domain.Request:
            '''
            importTextRequest processes the lines in a text block, extracts some metadata, and stores it as
            Request nodes in the KnowledgeGraph. This is used by the implementations of the abstract class
            AbstractContentRepo once they retrieve text content from their source. All of the metadata can be
            passed into this method, but will be overwritten by that discovered within the text (if any).
            '''
            request_text=""
            user=None
                    
            for line in text.splitlines():
                if line.startswith("date:"):
                    request_date = datetime.strptime(line.split(":",1)[1].strip(),'%m/%d/%Y').date()
                elif line.startswith("title:"):
                    title = line.split(":",1)[1].strip()
                elif line.startswith("user:"):
                    user_name = line.split(":",1)[1].strip()
                    user=kg.get_user_by_name(name=user_name)
                    if user is None: # Identified source not in KG so add it
                        usr=kg.add_source(name=user_name)
                else:
                    content_text=content_text.join(["\n",line])
                
            # write Content node to KG
            if user is None: # No user specified in file
                user=kg.get_user_by_name(name=default_user_name)
                if user is None: # Default does not exist in KG yet
                    user=kg.add_user(name=default_user_name, email=default_user_email)

            request=kg.add_request(user=user, title=title, text=content_text.strip())
            return request

class TxtFileContentRepo(AbstractContentRepo):
    '''
    Reads all the text files in the provided directory path. Creates a Source node for the directory and parses the text files.
    This extracts lines with property definitions (e.g., "date: 05/14/1960 08:00PM") and puts the rest into the text body.
    Creates a Content node, for each file and adds it to the knowledge graph.
    '''
    ingest_path:str = "."

    def __init__(self, kg:KnowledgeGraph, ingest_path="."):
        self.kg=kg
        self.ingest_path = ingest_path

    def describe(self) -> str:
        return "Text File Content (path = '{path}')".format(path=self.ingest_path)

    def importContentData(self) -> list[str]:
        _log.info("Importing Content from File System...")
        default_source_name = "File System Directory:{path}".format(path=self.ingest_path)
        list_of_uids = []

        # read in each text file from directory
        for filename in os.listdir(self.ingest_path):

            # ignore anything but files that end in .txt
            if os.path.splitext(filename)[1] != ".txt" or not os.path.isfile(os.path.join(self.ingest_path, filename)):
                continue

            # do not reimport articles that are already in the KG
            #if kg.get_content_by_url(url=filename) is not None:
            #    _log.info("Skipping content already in KG: {url}".format(url=filename))
            #    continue

            full_path=os.path.join(self.ingest_path, filename)
            with open(full_path) as f:
                text = f.read()
                url="file://{path}".format(path=os.path.abspath(full_path))
                content=self.importTextContent(kg=self.kg, text=text, url=url, default_source_name=default_source_name)
                list_of_uids.append(content.uid)

        return list_of_uids

class S3ContentRepo(AbstractContentRepo):

    bucket = None
    bucket_name=""
   
    def __init__(self, kg:KnowledgeGraph, bucket_name:str, aws_access_key_id:str, aws_secret_access_key:str):
        self.kg=kg
        session = boto3.Session( aws_access_key_id, aws_secret_access_key)
        s3 = session.resource('s3')
        self.bucket_name=bucket_name
        self.bucket = s3.Bucket(bucket_name)

    def describe(self) -> str:
        return "S3 Bucket Content ('{bucket}')".format(bucket=self.bucket_name)

    def importContentData(self) -> list[str]:
        _log.info("Importing Content from S3 Bucket...")
        default_source_name = "S3 Bucket:{path}".format(path=self.bucket_name)
        list_of_uids = []

        # read in each text file from directory
        for object in self.bucket.objects.all():
            # ignore anything but files that end in .txt
            if not object.key.endswith('txt'):
                continue

            url="https://{bucket_name}.s3.amazonaws.com/{object_name}".format(bucket_name=self.bucket_name, object_name=object.key)
            text=object.get()['Body'].read().decode()
            content=self.importTextContent(kg=self.kg, text=text, url=url, default_source_name=default_source_name)
            list_of_uids.append(content.uid)

        return list_of_uids


class RssContentRepo(AbstractContentRepo):
    feedname:str="unknown"

    def describe(self) -> str:
        return "RSS Feed Content ('{feed}')".format(feed=self.feed_name)
    
    def importContentData(self) -> list[str]:
        _log.info("Importing Content from RSS Feed...")
        raise NotImplementedError("RSS Feed Content Repo is not yet implemented.")

class MongoContentRepo(AbstractContentRepo):
    collection:str="unknown"

    def describe(self) -> str:
        return "MongoDB Content ('{collection}')".format(collection=self.collection)

    def importContentData(self) -> list[str]:
        _log.info("Importing Content from MongoDB...")
        raise NotImplementedError("MongoDB Content Repo is not yet implemented.")

class ContentRepo(AbstractContentRepo):
    repo:AbstractContentRepo

    def _configOrEnv(self, key:str, config:dict) -> str:
        value:str=config.get(key)
        if value is None:
            value = config.get(key.capitalize())
        if value is None:
            value = os.getenv(key)
        if value is None:
            value = os.getenv(key.capitalize())
        if value is None:
            raise TypeError("No value found for '{key}', which is required.".format(key=key))
        return value

    def __init__(self, kg:KnowledgeGraph, config:dict):
        if config is None:
            _log.warn("No config file passed to ContentRepo, defaulting to using environment variables.")
            config="{}"

        type = self._configOrEnv("repo_type",config);
        if type is None or type not in ['s3','files']:
            message=("RepoFactor requires a dictionary structure as a parameter that includes at least a 'repo_type' entry set to one of 's3' or 'file'.")
            _log.error(message)
            raise TypeError(message)
        
        match type:
            case "s3":
                bucket_name=self._configOrEnv("bucket",config)
                aws_access_key_id=self._configOrEnv("aws_access_key_id",config)
                aws_secret_access_key=self._configOrEnv("aws_secret_access_key",config)
                self.repo= S3ContentRepo(kg,bucket_name=bucket_name, aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key)
            case "files":
                ingest_path=self._configOrEnv("path",config)
                self.repo= TxtFileContentRepo(kg,ingest_path=ingest_path)
            case _:
                message:str = "Invalid 'type' config ('{type}').".format(type=type)
                _log.error(message)
                raise NotImplementedError(message)
        
    def importReferenceData(self) -> list[str]:
        message:str = "Importing Reference Data is not implemented yet."
        _log.error(message)
        raise NotImplementedError(message)

    def importRequestData(self) -> list[str]:
        message:str = "Importing Request Data is not implemented yet."
        _log.error(message)
        raise NotImplementedError(message)

    def importContentData(self) -> list[str]:
        if self.repo is None:
            _log.error("No to ContentRepo was configured.")
        _log.info("Importing Content from ContentRepo: {repo}.".format(repo=self.repo.describe()))
        return self.repo.importContentData()
    
    def importFeedbackData(self) -> list[str]:
        message:str = "Importing Feedback Data is not implemented yet."
        _log.error(message)
        raise NotImplementedError(message)

    def importEvaluationData(self) -> list[str]:
        message:str = "Importing Evaluation Data is not implemented yet."
        _log.error(message)
        raise NotImplementedError(message)

    def describe(self) -> str:
        if self.repo is None:
            _log.error("No to ContentRepo was configured.")
        return self.repo.describe()